==> audiochain-1.1.5-src.tgz: org/audiochain/model/AudioDeviceValue.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

import static org.audiochain.model.AudioProject.*;

import java.util.Collection;
import java.util.Collections;
import java.util.Formatter;
import java.util.LinkedHashSet;

/**
 * <p>
 * The AudioDeviceValue is a logical link to an AudioDataReader. It should have
 * a name that is meaningful to the user (for example Gain, Pan, Delay, etc).
 * The change of the value must change the state of the AudioDataReader this
 * AudioDeviceValue is linked with. To realize this, the
 * AudioDeviceValueListener can be used.
 */
abstract public class AudioDeviceValue implements Synchronizable {
	private String name;
	private Collection<AudioDeviceValueListener> listeners;
	final AudioDevice audioDevice;
	private String unit;
	private char symbol;

	public AudioDeviceValue(AudioDevice audioDevice, String name) {
		this.audioDevice = audioDevice;
		this.name = name;
	}

	void copyFrom(AudioDeviceValue from) {
		if (!getName().equals(from.getName())) {
			throw new IllegalArgumentException("The " + AudioDeviceValue.class.getSimpleName() + " must have the same name. This name '" + getName() + "', the other name to be copied from is '"
					+ from.getName() + "'.");
		}
		copy(from);
	}

	abstract protected void copy(AudioDeviceValue from);

	abstract public void setValueAsString(String stringValue);

	abstract public String getValueAsString();

	public String getName() {
		return name;
	}
	
	public void changeName(String name) {
		this.name = name;
	}

	protected boolean isEqual(Object o1, Object o2) {
		if (o1 == null && o2 == null) {
			return true;
		}
		return o1 != null && o1.equals(o2);
	}

	protected final void fireAudioDeviceValueChanged(String oldValue, String newValue) {
		audioDevice.setAudioDeviceValueLastChangeTime(getName(), System.currentTimeMillis());
		if (listeners != null) {
			for (AudioDeviceValueListener listener : new LinkedHashSet<AudioDeviceValueListener>(listeners)) {
				listener.audioDeviceValueChanged(audioDevice, this, oldValue, newValue);
			}
		}
	}

	public void addAudioDeviceValueListener(AudioDeviceValueListener listener) {
		if (listeners == null) {
			listeners = new LinkedHashSet<AudioDeviceValueListener>();
		}
		listeners.add(listener);
	}

	public void removeAudioDeviceValueListener(AudioDeviceValueListener listener) {
		if (listeners != null) {
			listeners.remove(listener);
			if (listeners.isEmpty()) {
				listeners = null;
			}
		}
	}

	void clearListeners() {
		if (listeners != null) {
			listeners.clear();
			listeners = null;
		}
	}

	public Collection<AudioDeviceValueListener> getListeners() {
		return Collections.unmodifiableCollection(listeners);
	}

	public AudioDevice getAudioDevice() {
		return audioDevice;
	}

	public String getUnit() {
		return unit;
	}

	public void setUnit(String unit) {
		this.unit = unit;
	}

	@Override
	public void synchronizeBy(Synchronizable source) {
		if (source instanceof AudioDeviceValue) {
			AudioDeviceValue sourceAudioDeviceValue = (AudioDeviceValue) source;
			long sourceTime = sourceAudioDeviceValue.audioDevice.getAudioDeviceValueLastChangeTime(name);
			long targetTime = audioDevice.getAudioDeviceValueLastChangeTime(name);
			if (greaterOrBothZero(sourceTime, targetTime) && sourceAudioDeviceValue.getName().equals(getName())) {
				setValueAsString(sourceAudioDeviceValue.getValueAsString());
				audioDevice.setAudioDeviceValueLastChangeTime(name, sourceTime);
			}
		}
	}

	@Override
	public String getIdentifier() {
		return name;
	}

	public String getPropertyName() {
		return getName().replace(' ', '-').replaceAll("-<", " <");
	}

	abstract public String getHelpString();

	public String getDisplayString() {
		StringBuilder sb = new StringBuilder();
		if (symbol != ' ') {
			sb.append(symbol);
		}
		sb.append(getValueAsString());
		if (unit != null) {
			sb.append(unit);
		}
		return sb.toString();
	}

	protected final String format(String defaultValue, String optionsAndRanges) {
		StringBuilder sb = new StringBuilder();
		Formatter format = new Formatter(sb);
		format.format("    %-25s %-15s %-15s%n", getPropertyName(), defaultValue, optionsAndRanges);
		return sb.toString();
	}

	protected final String format(String helpString) {
		StringBuilder sb = new StringBuilder();
		Formatter format = new Formatter(sb);
		format.format("    %-25s %-15s%n", getPropertyName(), helpString);
		return sb.toString();
	}

	@Override
	public final String toString() {
		return super.toString();
	}

	public char getSymbol() {
		return symbol;
	}

	public void setSymbol(char symbol) {
		this.symbol = symbol;
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/model/AudioDeviceValue.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

import static org.audiochain.model.AudioProject.greaterOrBothZero;

import java.util.Collection;
import java.util.Collections;
import java.util.Formatter;
import java.util.LinkedHashSet;

/**
 * <p>
 * The AudioDeviceValue is a logical link to an AudioDataReader. It should have
 * a name that is meaningful to the user (for example Gain, Pan, Delay, etc).
 * The change of the value must change the state of the AudioDataReader this
 * AudioDeviceValue is linked with. To realize this, the
 * AudioDeviceValueListener can be used.
 */
abstract public class AudioDeviceValue implements Synchronizable {
	private String name;
	private Collection<AudioDeviceValueListener> listeners;
	final AudioDevice audioDevice;
	private String unit;
	private char symbol;

	public AudioDeviceValue(AudioDevice audioDevice, String name) {
		this.audioDevice = audioDevice;
		this.name = name;
	}

	void copyFrom(AudioDeviceValue from) {
		if (!getName().equals(from.getName())) {
			throw new IllegalArgumentException("The " + AudioDeviceValue.class.getSimpleName() + " must have the same name. This name '" + getName() + "', the other name to be copied from is '"
					+ from.getName() + "'.");
		}
		copy(from);
	}

	abstract protected void copy(AudioDeviceValue from);

	abstract public void setValueAsString(String stringValue);

	abstract public String getValueAsString();

	public String getName() {
		return name;
	}
	
	public void changeName(String name) {
		this.name = name;
	}

	protected boolean isEqual(Object o1, Object o2) {
		if (o1 == null && o2 == null) {
			return true;
		}
		return o1 != null && o1.equals(o2);
	}

	protected final void fireAudioDeviceValueChanged(String oldValue, String newValue) {
		audioDevice.setAudioDeviceValueLastChangeTime(getName(), System.currentTimeMillis());
		if (listeners != null) {
			for (AudioDeviceValueListener listener : new LinkedHashSet<AudioDeviceValueListener>(listeners)) {
				listener.audioDeviceValueChanged(audioDevice, this, oldValue, newValue);
			}
		}
	}

	public void addAudioDeviceValueListener(AudioDeviceValueListener listener) {
		if (listeners == null) {
			listeners = new LinkedHashSet<AudioDeviceValueListener>();
		}
		listeners.add(listener);
	}

	public void removeAudioDeviceValueListener(AudioDeviceValueListener listener) {
		if (listeners != null) {
			listeners.remove(listener);
			if (listeners.isEmpty()) {
				listeners = null;
			}
		}
	}

	void clearListeners() {
		if (listeners != null) {
			listeners.clear();
			listeners = null;
		}
	}

	public Collection<AudioDeviceValueListener> getListeners() {
		return Collections.unmodifiableCollection(listeners);
	}

	public AudioDevice getAudioDevice() {
		return audioDevice;
	}

	public String getUnit() {
		return unit;
	}

	public void setUnit(String unit) {
		this.unit = unit;
	}

	@Override
	public void synchronizeBy(Synchronizable source) {
		if (source instanceof AudioDeviceValue) {
			AudioDeviceValue sourceAudioDeviceValue = (AudioDeviceValue) source;
			long sourceTime = sourceAudioDeviceValue.audioDevice.getAudioDeviceValueLastChangeTime(name);
			long targetTime = audioDevice.getAudioDeviceValueLastChangeTime(name);
			if (greaterOrBothZero(sourceTime, targetTime) && sourceAudioDeviceValue.getName().equals(getName())) {
				setValueAsString(sourceAudioDeviceValue.getValueAsString());
				audioDevice.setAudioDeviceValueLastChangeTime(name, sourceTime);
			}
		}
	}

	@Override
	public String getIdentifier() {
		return name;
	}

	public String getPropertyName() {
		return getName().replace(' ', '-').replaceAll("-<", " <");
	}

	abstract public String getHelpString();

	public String getDisplayString() {
		StringBuilder sb = new StringBuilder();
		if (symbol != ' ') {
			sb.append(symbol);
		}
		sb.append(getValueAsString());
		if (unit != null) {
			sb.append(unit);
		}
		return sb.toString();
	}

	protected final String format(String defaultValue, String optionsAndRanges) {
		StringBuilder sb = new StringBuilder();
		Formatter format = new Formatter(sb);
		format.format("    %-25s %-15s %-15s%n", getPropertyName(), defaultValue, optionsAndRanges);
		String ret = sb.toString();
		format.close();
		return ret;
	}

	protected final String format(String helpString) {
		StringBuilder sb = new StringBuilder();
		Formatter format = new Formatter(sb);
		format.format("    %-25s %-15s%n", getPropertyName(), helpString);
		String ret = sb.toString();
		format.close();
		return ret;
	}

	@Override
	public final String toString() {
		return super.toString();
	}

	public char getSymbol() {
		return symbol;
	}

	public void setSymbol(char symbol) {
		this.symbol = symbol;
	}
}
